/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mouse Gestures for Mozilla.
 *
 * The Initial Developer of the Original Code is Jens Bannmann.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Jens Bannmann <jens.b@web.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var windowType; // type of the window this mozgest instance is working on
var activationPref; // preference name
var contentAreaGetter;
var contextMenuID;
var messageSetter;      // js code to display a message; alternative:
var statusbarcontainer; // id of the element a statusbar is appended to
var mgNavtoolbox;


function determineWindowType() {
  var rdfUrl = "chrome://mozgest/content/windowTypes.rdf"

  // get our datasource
  var dsource = RDFService.GetDataSourceBlocking(rdfUrl);

  // instantiate a container object and its root node
  var mappingsContainer = Components.classes['@mozilla.org/rdf/container;1'].createInstance()
                .QueryInterface(Components.interfaces.nsIRDFContainer);
  var rootnode = RDFService.GetResource("urn:mozgest:windows:supported");

  try {
    mappingsContainer.Init(dsource, rootnode);
  } catch (e) {
    dump("Fatal error - could not read windowTypes.rdf: " + e);
  }

  windowType = false;

  var enumerator = mappingsContainer.GetElements();
  while (enumerator.hasMoreElements()) {
    var typeDef = enumerator.getNext();

    if (typeDef) {
      // get the type id
      var t = getPropertyValue(dsource, typeDef, mgNs + "type");
      // default activationpref is windowtype, but may be set differently
      var ap = getPropertyValue(dsource, typeDef, mgNs + "depends") || t;
      // set all windowtype prefs to true if not present
      mgGetPref(ap, "Bool", true);

      if (getPropertyValue(dsource, typeDef, mgNs + "location")
          == document.location.href) {
        windowType = t;
        activationPref = ap;
        contentAreaGetter = getPropertyValue(dsource, typeDef, mgNs + "contentarea");
        contextMenuID = getPropertyValue(dsource, typeDef, mgNs + "contextmenu");
        messageSetter = getPropertyValue(dsource, typeDef, mgNs + "showmsg");
        statusbarcontainer = getPropertyValue(dsource, typeDef, mgNs + "statusbarcontainer");
      }
    }
  }

  if (windowType == false) {
    dump("Fatal error: could not identify overlayed window in windowTypes.rdf");
  }

  if (statusbarcontainer != null && !document.getElementById("mgStatusbar")) {
    dockElementAt("statusbar", "statusbarpanel", "mgStatusbar",
                  new Array("flex", "label"),
                  new Array("1", ""),
                  document.getElementById(statusbarcontainer));
  }

  mgNavtoolbox = document.getElementById("navigator-toolbox");
  if (mgNavtoolbox) {
    windowType = (mgNavtoolbox.getAttribute("hidden") == "true") ? "browser_printpreview"
                                                                 : "browser";
    // get notified when user enters or leaves print-preview
    mgNavtoolbox.addEventListener("DOMAttrModified", navToolboxChanged, true);
  }
}

// since most browser commands are disabled or buggy in print preview,
// we switch to a different gesture set, although it's the same window
function navToolboxChanged(event) {
  if (event.target == mgNavtoolbox && event.attrName == "hidden") {
    windowType = (event.newValue == "true") ? "browser_printpreview" : "browser";
    resetGesturesTable();
    readRDFGestureTable("window");
    readRDFGestureTable(windowType);
  }
}

function mgGetContextMenu() {
  return document.getElementById(contextMenuID);
}


function mgGetContentArea() {
  return eval(contentAreaGetter);
}

function mgMsg(msgStr) {
  if (messageSetter != null) {
    eval(messageSetter);
  } else if (statusbarcontainer != null) {
    document.getElementById("mgStatusbar").setAttribute("label", msgStr);
  }
}

// mozgestIsInContent is called from startGesture to check whether the click has 
// occured in content area
function mozgestIsInContent(e){
  // collect all ancestor element names
  var found = new Array();
  var cur = e.originalTarget;
  while (cur != null && cur != mgGetContentArea()) {
    found[cur.nodeName.toLowerCase()] = true;
    cur = cur.parentNode;
  }

  // prevent gestures over scrollbars or input elements
  if (found["scrollbar"] || found["select"] || found["input"] ||
      found["textarea"] || found["textbox"]) {
    return false;
  }

  // do not allow gestures on tab toolbar
  if (mgGetContentArea().nodeName == "tabbrowser"
      && !(found["#document"] || found["browser"])) {
    return false;
  }

  // if we get here, all is okay
  return true;
}


function mgIsEnabled() {
  return mgGetPref(activationPref, "Bool", true);
}
